import networkx as nx

from nxviz.plots import BasePlot


def test_initialization():
    """
    Tests initialization of plot object.
    """
    n_nodes = 10
    G = nx.erdos_renyi_graph(n=n_nodes, p=0.3)  # noqa

    b = BasePlot(graph=G)

    assert len(b.nodes) == len(G.nodes())


def make_graph_for_grouping():

    nodelist = [
        ("Andrew", {"affiliation": "MIT", "year": 5, "score": -2.0}),
        ("Felipe", {"affiliation": "Broad", "year": 4, "score": 0.0}),
        ("Tom", {"affiliation": "Harvard", "year": 2, "score": 1.0}),
        ("Liz", {"affiliation": "Broad", "year": 3, "score": 2.0}),
        ("Lily", {"affiliation": "MIT", "year": 2, "score": 3.0}),
        ("Jessica", {"affiliation": "MIT", "year": 2, "score": 0.0}),
    ]

    G = nx.Graph()  # noqa
    G.add_nodes_from(nodelist)
    return G


def make_graph_for_edges():

    nodelist = [("a"), ("b"), ("c"), ("d"), ("e"), ("f")]
    edgelist = [
        ("a", "b", {"weight": 0.1}),
        ("a", "c", {"weight": 0.2}),
        ("b", "d", {"weight": 0.6}),
        ("c", "d", {"weight": 0.7}),
        ("e", "d", {"weight": 0.8}),
        ("e", "f", {"weight": 1.0}),
    ]

    G = nx.Graph()  # noqa
    G.add_nodes_from(nodelist)
    G.add_edges_from(edgelist)
    return G


def test_init_group_nodes():
    """
    Tests initialization with grouping of nodes.

    This only tests that the nodes are ordered correctly when sorted on the
    `node_grouping` key.
    """

    G = make_graph_for_grouping()  # noqa
    b = BasePlot(graph=G, node_grouping="affiliation")

    assert b.nodes == [
        n
        for n, d in sorted(
            G.nodes(data=True), key=lambda x: x[1]["affiliation"]
        )
    ]


def test_init_sort_and_group_nodes():
    """
    Tests initialization with sorting and grouping of nodes.

    This tests that the nodes are ordered correctly when first grouped on the
    `node_grouping` key, and then sorted within each group on the `node_order`
    key.
    """
    G = make_graph_for_grouping()  # noqa

    b = BasePlot(graph=G, node_grouping="affiliation", node_order="year")

    assert b.nodes == [
        n
        for n, d in sorted(
            G.nodes(data=True),
            key=lambda x: (x[1]["affiliation"], x[1]["year"]),
        )
    ]


def test_init_sort_nodes():
    """
    Tests initialization with sorting of nodes.

    This tests that the nodes are ordered correctly when sorted on the
    "node_order" key.
    """

    G = make_graph_for_grouping()  # noqa

    b = BasePlot(graph=G, node_order="year")

    assert b.nodes == [
        n for n, d in sorted(G.nodes(data=True), key=lambda x: x[1]["year"])
    ]


def test_init_data_types():
    """
    Checks that the data_types dictionary is initialized correctly.
    """

    G = make_graph_for_grouping()  # noqa

    b = BasePlot(
        graph=G, data_types={"year": "ordinal", "affiliation": "categorical"}
    )
    assert isinstance(b.data_types, dict)


def test_init_node_colors():
    """
    Does two checks:
    1. If node_color is not passed in as a keyword argument, check that
       self.node_colors is a list of 'blue', of length (number of nodes).
    2. If node_color is passed in as a keyword argument, check that
       self.node_colors is a list with more than one element.
    """
    G = make_graph_for_grouping()  # noqa
    b = BasePlot(graph=G, node_color="year")
    assert len(set(b.node_colors)) > 1

    G = make_graph_for_grouping()  # noqa
    b = BasePlot(graph=G)
    assert len(set(b.node_colors)) == 1


def test_init_edge_colors():
    """
    Does two checks:
    1. If edge_color is passed in as a keyword argument, check that
       self.edge_colors is a list with more than one element.
    2. If edge_color is not passed in as a keyword argument, check that
       self.edge_colors is a list of 'black', of length 0.
    """
    G = make_graph_for_edges()  # noqa
    b = BasePlot(graph=G, edge_color="weight")
    assert len(set(b.edge_colors)) > 1

    G = make_graph_for_grouping()  # noqa
    b = BasePlot(graph=G)
    assert len(set(b.edge_colors)) == 0



"""
Utility geometry functions that can help with drawing to screen.
"""
import numpy as np

from .polcart import to_cartesian


def node_theta(nodelist, node):
    """
    Maps node to Angle.

    :param nodelist: Nodelist from the graph.
    :type nodelist: list.
    :param node: The node of interest. Must be in the nodelist.
    :returns: theta -- the angle of the node in radians.
    """
    assert len(nodelist) > 0, "nodelist must be a list of items."
    assert node in nodelist, "node must be inside nodelist."

    i = nodelist.index(node)
    theta = -np.pi + i * 2 * np.pi / len(nodelist)

    return theta


def group_theta(node_length, node_idx):
    """
    Returns an angle corresponding to a node of interest.

    Intended to be used for placing node group labels at the correct spot.

    :param float node_length: total number of nodes in the graph.
    :param int node_idx: the index of the node of interest.
    :returns: theta -- the angle of the node of interest in radians.
    """
    theta = -np.pi + node_idx * 2 * np.pi / node_length
    return theta


def text_alignment(x, y):
    """
    Align text labels based on the x- and y-axis coordinate values.

    This function is used for computing the appropriate alignment of the text
    label.

    For example, if the text is on the "right" side of the plot, we want it to
    be left-aligned. If the text is on the "top" side of the plot, we want it
    to be bottom-aligned.

    :param x, y: (`int` or `float`) x- and y-axis coordinate respectively.
    :returns: A 2-tuple of strings, the horizontal and vertical alignments
        respectively.
    """
    if x == 0:
        ha = "center"
    elif x > 0:
        ha = "left"
    else:
        ha = "right"
    if y == 0:
        va = "center"
    elif y > 0:
        va = "bottom"
    else:
        va = "top"

    return ha, va


def get_cartesian(r, theta):
    """
    Returns the cartesian (x,y) coordinates of (r, theta).

    :param r: Real-valued radius.
    :type r: int, float.
    :param theta: Angle
    :type theta: int, float.
    :returns: to_cartesian(r, theta)
    """
    return to_cartesian(r, theta)


def correct_negative_angle(angle):
    """
    Corrects a negative angle to a positive one.

    :param angle: The angle in radians.
    :type angle: float
    :returns: `angle`, corrected to be positively-valued.
    """
    if angle < 0:
        angle = 2 * np.pi + angle
    else:
        pass

    return angle


def circos_radius(n_nodes, node_r):
    """
    Automatically computes the origin-to-node centre radius of the Circos plot
    using the triangle equality sine rule.

    a / sin(A) = b / sin(B) = c / sin(C)

    :param n_nodes: the number of nodes in the plot.
    :type n_nodes: int
    :param node_r: the radius of each node.
    :type node_r: float
    :returns: Origin-to-node centre radius.
    """
    A = 2 * np.pi / n_nodes  # noqa
    B = (np.pi - A) / 2  # noqa
    a = 2 * node_r
    return a * np.sin(B) / np.sin(A)
